import { addImportsSources, addComponent, useNuxt, defineNuxtModule } from '@nuxt/kit';

const libraryName = "vue-map-ui";
const allComponents = ["VMap", "VMapTileLayer", "VMapOsmTileLayer"];
const allImports = ["useMap", "useLayer"];
const defaults = {
  components: allComponents,
  imports: allImports,
  autoImports: true,
  leafletStyles: true,
  libraryStyles: true,
  normalizeStyles: true
};

function resolveImports(config) {
  const { imports } = config;
  const allImports = new Set(imports);
  addImportsSources({
    from: libraryName,
    imports: [...allImports]
  });
}

function isArray(value) {
  return Array.isArray(value);
}
function toArray(value) {
  return isArray(value) ? value : [value];
}

function resolveComponents(config) {
  const { components } = config;
  const allComponents = new Set(components);
  for (const item of allComponents) {
    const [name, alias] = toArray(item);
    addComponent({
      export: name,
      name: alias || name,
      filePath: libraryName
    });
  }
}

function resolveStyles(config) {
  const nuxt = useNuxt();
  const enabled = isArray(nuxt.options.css);
  if (config.leafletStyles) {
    enabled && nuxt.options.css.push("leaflet/dist/leaflet.css");
  }
  if (config.normalizeStyles) {
    enabled && nuxt.options.css.push(`${libraryName}/dist/normalize.css`);
  }
  if (config.libraryStyles) {
    enabled && nuxt.options.css.push(`${libraryName}/dist/style.css`);
  }
}

const module = defineNuxtModule({
  meta: {
    name: libraryName,
    configKey: "vueMap"
  },
  defaults,
  setup(_options, nuxt) {
    const options = _options;
    resolveStyles(options);
    nuxt.options.imports.autoImport !== false && options.autoImports && resolveImports(options);
    nuxt.options.components !== false && resolveComponents(options);
  }
});

export { module as default };
